from bson import json_util
from flask import Flask, request
from pymongo import MongoClient

from modele.contenuDao import ContenuDao
from modele.demande_pubDao import DemandePubDao
from modele.encoursDao import EncoursDao
from modele.evaluationDao import EvaluationDao
from modele.utilisateurDao import UtilisateurDao
import json

app = Flask(__name__)

contenu_dao = ContenuDao()
utilisateur_dao = UtilisateurDao()
demande_pub_Dao = DemandePubDao()
evaluation_dao = EvaluationDao()
en_cours_dao = EncoursDao()

client = MongoClient('localhost', 27017)

base_donnees = client.squelette


## *********************************************************************************************************************
## Routage utilisateur : 1 post, 1 get, 2 patch
@app.route('/newUser', methods=['POST'])
def creer_utilisateur():
    utilisateur_dao.ajouter_utilisateur(request.json["login"])
    return "Utilisateur créé"


@app.route('/checkUser', methods=['POST'])
def verifier_utilisateur():
    return str(utilisateur_dao.presence_utilisateur(request.json["login"]))


@app.route('/changePassword', methods=['PATCH'])
def changer_mdp():
    pass


@app.route('/ajouterTiers', methods=['POST'])
def ajouter_tiers():
    contenu_dao.creer_tiers(request.json["auteur"], request.json["id_tiers"], 1)
    return "tiers ajouté"


## *********************************************************************************************************************
## Routage contenus : beaucoup de fonctions
@app.route("/newContenu", methods=['POST'])
def ajouter_contenu():
    auteur = request.json["auteur"]
    nom_fichier = request.json["nom_fichier"]
    commit = request.json["commit"]
    contenu_dao.ajouter_contenu(auteur, nom_fichier, commit)
    return "Contenu ajouré"


@app.route("/getContenu/<id>", methods=['GET'])
def get_contenu_par_id(id):
    return json.dumps(list(contenu_dao.get_contenu_par_id(int(id))), default=json_util.default)


@app.route("/getContenuPanier/<auteur>", methods=['GET'])
def get_contenu_panier(auteur):
    print(contenu_dao.get_contenus(auteur, True, True, True))
    return json.dumps(contenu_dao.get_contenus(auteur, True, True, True), default=json_util.default)


@app.route("/deletePerso/<id_perso>", methods=['DELETE'])
def supprimer_perso(id_perso):
    contenu_dao.supprimer_contenu(id_perso)


@app.route("/deleteTiers/<id_tiers>", methods=['DELETE'])
def supprimer_tiers(id_tiers):
    contenu_dao.supprimer_tiers(id_tiers, request.args.get('auteur'), request.args.get('version'))


@app.route("/newContenu", methods=['POST'])
def creer_nouveau_contenu():
    pass


@app.route("/majContenu", methods=['PATCH'])
def maj_contenu():
    pass


@app.route("/allContenus", method=['GET'])
def get_tous_les_contenus():
    return json.dumps(list(contenu_dao.get_tous_les_contenus()), default=json_util.default)


@app.route("/isContenuAnonyme/<id_contenu>", methods=['GET'])
def is_contenu_anonyme(id_contenu):
    return str(contenu_dao.is_anonyme(id_contenu))


## *********************************************************************************************************************
## Gestion des Contenus en Cours


@app.route("/getEnCours/<id_en_cours>", methods=['GET'])
def get_contenu_en_cours(id_en_cours):
    return json.dumps(list(en_cours_dao.get_contenu_en_cours(id_en_cours)), default=json_util.default)


@app.route("/getAuteurOrigineEnCours/<id_en_cours>", methods=['GET'])
def get_contenu_en_cours(id_en_cours):
    return json.dumps(list(en_cours_dao.get_auteur_origine_contenu_en_cours(id_en_cours)), default=json_util.default)


@app.route("/isContenuDemandePublication/<id_en_cours>/<auteur>", method=['GET'])
def is_contenu_demande_publication(id_en_cours, auteur):
    return str(en_cours_dao.is_en_cours_demande(id_en_cours, auteur))


@app.route("/supprimerEnCours/<id_en_cours>", methods=['DELETE'])
def supprimer_en_cours(id_en_cours):
    en_cours_dao.supprimer_contenu_en_cours(id_en_cours)


@app.route("/newEnCours", methods=['POST'])
def creer_en_cours():
    en_cours_dao.creer_en_cours(request.args.get('id_contenu'), request.args.get('version'),
                                request.args.get('branche'))
    return "En Cours créé"


## *********************************************************************************************************************
## Gestion des Publications

@app.route("/demandePublication", methods=['POST'])
def demande_de_publication():
    demande_pub_Dao.creer_demande_publication(request.args.get('id_contenu'), request.args.get('id_contenu_en_cours'),
                                              request.args.get('auteur_en_cours'), request.args.get('auteur'))


@app.route("/demandePublication/<auteur>/<emise>", method=['GET'])
def get_demandes_pub(auteur, emise):
    return demande_pub_Dao.get_demandes_pub(auteur, emise)


@app.route("/demandePublication/<id_publication>", method=['DELETE'])
def get_demandes_pub(id_publication):
    demande_pub_Dao.supprimer_publication(id_publication)


## *********************************************************************************************************************
## Gestion des Evaluations

@app.route("/newEval", methods=['POST'])
def creer_evaluation():
    evaluation_dao.creer_evaluation(request.args.get('contenus'), request.args.get('auteur'))
    return "evaluation créée"


@app.route("/getEval/<auteur>/<id_evaluation>", methods=['GET'])
def get_evaluation(auteur, id_evaluation):
    return json.dumps(list(evaluation_dao.get_evaluations(auteur, id_evaluation)), default=json_util.default)


@app.route("/getEval/<auteur>", methods=['GET'])
def get_all_evaluations(auteur):
    return json.dumps(list(evaluation_dao.get_evaluations(auteur, 0)), default=json_util.default)


@app.route("/deleteEval", methods=['DELETE'])
def supprimer_evaluation(id_evaluation):
    evaluation_dao.supprimer_evaluation(id_evaluation)


if __name__ == '__main__':
    app.run()
