from flask_pymongo import PyMongo

from pymongo import MongoClient
from random import randrange
from datetime import datetime
import os
# from UtilsGit import UtilsGit
from pathlib import Path
import string

from modele.encoursDao import EncoursDao


class ContenuDao:
    client = MongoClient('localhost', 27017)
    chemin = "/home/passerat/Stage/Squelette/"

    ## extraction d'une base de données. Si elle n'existe pas, elle esst créée
    base_donnees = client.squelette

    def ajouter_contenu(self, auteur, nom_fichier, commit):
        self.base_donnees.contenu.insert_one(
            {"id": 1, "auteur": auteur, "note": randrange(0, 10, 1),
             "fichier": "dossier" + nom_fichier + "/" + nom_fichier,
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "popularite": 0, "commit": commit},
             "tags_banque_fixes": ["tag1", "tag2", "tag3", "tag4", "tag5"],
             "tags_banque_libres": ["tagl1", "tagl2", "tagl3", "tagl4", "tagl5"], "type": " ", "source": " "})

    ## recherche d'un contenu par id (pour affichage de son contenu)
    # @param id l'id du contenu recherché
    def get_contenu_par_id(self, id):
        return self.base_donnees.contenu.find({"id": id})

    def get_contenu_avec_id_et_version(self, id_contenu, version):

        # a décommenter quand git sera actif
        # Utils.changer_version(idContenu, version, "")
        return self.base_donnees.contenu.find_one({"id": id_contenu})
        # return Utils.statdb.contenu.find_one({"$and": [
        #     {"id": idContenu},
        #     {"$or": [{"versionEnCours.numero": version}, {"historique.numero": version}]}
        # ]
        # })

    ## recherche des contenus pour un auteur donné (son 'panier')
    # param
    def get_contenus(self, auteur, perso, tiers, en_cours):

        contenus = list()

        if tiers:
            id_tiers = self.get_contenus_tiers(auteur)
            for (id, version) in id_tiers:
                contenus.append(
                    self.get_contenu_avec_id_et_version(int(id['contenu']), int(version['version'])))

        if perso:
            curseur_perso = self.get_contenus_persos(auteur)
            for contenu in curseur_perso:
                contenus.append(contenu)

        if en_cours:
            curseur_en_cours = EncoursDao().get_en_cours(auteur)
            for contenu in curseur_en_cours:
                contenus.append(contenu)
        # on garde le développement de l'évaliation pour plus tard
        # return contenus_persos + contenus_en_cours + contenus_tiers
        return contenus



    def creer_tiers(self, auteur, id_contenu, version):
        self.base_donnees.statdb.utilisateur.update({"nom": auteur}, {
            "$push": {
                "tiers": [{"contenu": int(id_contenu)}, {"version": version}]}})

    def get_contenus_persos(self):
        return self.base_donnees.contenu.find({"auteur": auteur})

    def get_contenus_tiers(self, auteur):
        return self.base_donnees.statdb.utilisateur.find_one({"nom": auteur})["tiers"]

    ## l'auteur 'renie' son contenu, qui reste disponible dans le panier de ceux qui le possédent
    def dissocier_contenu(self, id_contenu):
        self.base_donnees.contenu.update({"id": id_contenu}, {"$set": {"auteur": "Aucun"}})

    ## l'auteur 'renie' son contenu, mais vu que personne d'autre ne l'a il est définitivement supprimé
    def supprimer_contenu(self, id_contenu):
        self.base_donnees.db.contenu.delete_one({"id": id_contenu})

    def supprimer_tiers(self, id_contenu, auteur, version):
        self.base_donnees.statdb.utilisateur.update({"nom": auteur},
                                                    {"$pull": {
                                                        "tiers": [{"contenu": id_contenu}, {"version": version}]}})

    def existe_en_tiers(self, id_contenu):
        return self.base_donnees.statdb.utilisateur.find(
            {"tiers": {"$elemMatch": {"$elemMatch": {"contenu": id_contenu}}}}).count() != 0

    def creer_nouveau_contenu(self, chemin, auteur_en_cours, idContenuEnCours):
        # print("chemin : ", chemin)

        # hard codé pour l'exemple, sera adaptatif dans le futur
        max_id = 3

        dossierFichier = "dossierContenu" + str(max_id) + "/"
        cheminFichier = self.chemin + dossierFichier
        nomFichier = chemin[chemin.rfind("/") + 1:]

        # print("Chemin fichier : ", cheminFichier)
        # print("Dossier Fichier : ", dossierFichier)
        # print("NomFichier :", nomFichier)
        Path(cheminFichier).mkdir(parents=True, exist_ok=True)
        os.system("cp " + chemin + " " + cheminFichier)
        os.system("mv " + cheminFichier + nomFichier + " " + cheminFichier + "Contenu" + str(max_id))
        # exit(0)
        #commit = UtilsGit.commit(cheminFichier, "Contenu" + str(max_id), "Version 1")
        '''
        self.base_donnees.statdb.contenu.insert_one(
            {"id": max_id + 1, "auteur": auteur_en_cours, "note": randrange(0, 10, 1),
             "fichier": "dossierContenu" + str(max_id) + "/Contenu" + str(max_id),
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "popularite": 0, "commit": commit},
             "tags_fixes": {"1": "tag1", "2": "tag1", "3": "tag1", "4": "tag1",
                            "5": "tag1"},
             "tags": {"1": "tag1", "2": "tag1", "3": "tag1", "4": "tag1", "5": "tag1"}})

        self.base_donnees.supprimer_contenu_en_cours(idContenuEnCours)
        '''

    def get_chemin_du_contenu(self, idContenu):
        return self.chemin + self.base_donnees.contenu.find_one({"id": idContenu})["fichier"]

    def get_version_tiers_avec_id(self, idContenu, auteur):
        tiers = (self.base_donnees.utilisateur.find_one({"nom": auteur})["tiers"])
        for i in range(len(tiers)):
            tmp = tiers[i]
            if int(tmp[0]["contenu"]) == idContenu:
                return int(tmp[1]["version"])
        return 0

    def maj_contenu(self, chemin, idContenu, nomBranche, auteur=None):

        repertoire = chemin[:chemin.rfind("/") + 1]
        #fin = s.affichage(chemin)
        version = int(fin[-1])

        # Mise à jour du répertoire git
        #idCommit = UtilsGit.mergeVersion(repertoire, nomBranche)

        # Mise à jour des métadonnées - ok
        version_en_cours = self.base_donnees.statdb.contenu.find_one({"id": idContenu})["versionEnCours"]
        self.base_donnees.statdb.contenu.update_one({"id": int(idContenu)}, {"$push": {"historique": version_en_cours}})
        self.base_donnees.statdb.contenu.update_one({"id": int(idContenu)}, {
            "$set": {"versionEnCours": {"numero": version, "date": datetime.now().strftime("%d/%m/%y"),
                                        "popularite": 0, "commit": idCommit}}})
        if auteur is not None:
            self.base_donnees.statdb.contenu.update_one({"id": int(idContenu)}, {
                "$set": {"auteur": auteur}})

    def consulter_un_contenu(self, id, auteur):
        # Créer un répertoire au nom de l'auteur (ou l'utiliser si il existe déja)
        auteur = auteur.lower().replace(" ", "_")
        repertoire_auteur = self.chemin + auteur + "/"
        Path(repertoire_auteur).mkdir(parents=True, exist_ok=True)
        # Copier le fichier lié au contenu dans ce répertoire
        chemin = Utils.get_chemin_du_contenu(id)
        nom_fichier = chemin[chemin.rfind("/") + 1:]
        os.system("cp '" + chemin + "'  '" + repertoire_auteur + nom_fichier + "'")
        #self.affichage(chemin)

        ## détruire un fichier temporaire

    def modifier_contenu(self, idContenu, auteur, enCours=False):
        # le chemin du fichier restera tjrs le même
        contenu = self.base_donnees.statdb.contenu.find_one({"id": int(idContenu)})
        self.base_donnees.modification_fichier(self.chemin + contenu["fichier"], idContenu, auteur, not enCours)

    def detruire_fichier_tmp(self, id, auteur):
        auteur = auteur.lower().replace(" ", "_") + "/"
        chemin = self.get_chemin_du_contenu(id)
        nom_fichier = chemin[chemin.rfind("/") + 1:]
        os.system("rm '" + self.chemin + auteur + nom_fichier + "'")

    def get_max_id_contenus(self):
        ## Fonction permettant de récupérer l'id max présent en base (pour pouvoir incrémenter un nouveau contenu).
        element = self.base_donnees.statdb.contenu.find().sort("id", -1).limit(1)

        for elt in element:
            # print("id : ", elt["id"])
            return int(elt["id"])

    def is_anonyme(self, idContenu):
        # print("Is anonyme : ", Utils.statdb.contenu.find_one({"id": idContenu})["auteur"])
        return self.base_donnees.contenu.find_one({"id": idContenu})["auteur"] == "Aucun"

    def get_tous_les_contenus(self):
        return self.base_donnees.contenu.find()

    def is_tiers_auteur(self, auteur, idContenu):
        return self.get_version_tiers_avec_id(idContenu, auteur) != 0
